unit rmoeLed;

interface

uses
  Classes, Controls, Graphics,
  Dialogs, Sysutils, // only for show message during debug
  rmBase;

type

{------------------------------------------------------------------------}
{   TrmoeCustomLed, derived from TrmBasicShape (rmBase.pas unit)
{------------------------------------------------------------------------}

  TrmoeCustomLed = class(TrmBasicShape)
  public
    property Shape;
    property Width;
    property Height;
    property BorderStyle;
    property BorderWidth;
    property BorderColor;
    property ActiveColor;
    property InactiveColor;
    property StandByColor;
    property HilightColor;
    property ShaddowColor;
    property Active;
    property StandBy;
    property Visible;
  end;


{------------------------------------------------------------------------}
{   TLedSupport, derived from TrmBasicShape (rmBase.pas unit)
{------------------------------------------------------------------------}

  TLedSupport = class(TrmBasicShape)
  published
    property Shape;
    property BorderStyle;
    property BorderWidth;
    property BorderColor;
    property ActiveColor;
    property InactiveColor;
    property StandByColor;
    property HilightColor;
    property ShaddowColor;
  end;


{------------------------------------------------------------------------}
{   TLedLight, derived from TrmBasicShape (rmBase.pas unit)
{------------------------------------------------------------------------}

  TLedLight = class(TrmBasicShape)
  published
    property Shape;
    property Width;
    property Height;
    property BorderStyle;
    property BorderWidth;
    property BorderColor;
    property ActiveColor;
    property InactiveColor;
    property StandByColor;
    property HilightColor;
    property ShaddowColor;
  end;


{------------------------------------------------------------------------
   TrmoeBasicLed, derived from TGraphicControl (controls.pas)
   Component registration in rmoeReg.pas unit
{------------------------------------------------------------------------}

  TrmoeBasicLed = class(TGraphicControl)
  private
    FBmpBuffer: TBitmap;
    FActive: boolean;
    FStandBy: boolean;
    FLight: TLedLight;
    FSupport: TLedSupport;
    FLightHorzOffset: integer;
    FLightVertOffset: integer;
    procedure SetActive(value: boolean);
    procedure SetStandBy(value: boolean);
    procedure SetLightHorzOffset(value: integer);
    procedure SetLightVertOffset(value: integer);
  protected
    procedure Paint; override;
    procedure UpdateBmpBuffer;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Active: boolean read FActive write SetActive;
    property StandBy: boolean read FStandBy write SetStandBy;
    property Light: TLedLight read FLight write FLight;
    property Support: TLedSupport read FSupport write FSupport;
    property LightHorzOffset: integer read FLightHorzOffset write SetLightHorzOffset;
    property LightVertOffset: integer read FLightVertOffset write SetLightVertOffset;
    property PopupMenu;
    property Hint;
    property ShowHint;
    property Tag;
    property Visible;
    property OnClick;
  end;



implementation


{------------------------------------------------------------------------
   TrmoeBasicLed, derived from TrmoeCustomBasicLed (rmoeLed.pas unit)
{------------------------------------------------------------------------}

constructor TrmoeBasicLed.Create(Owner: TComponent);
begin
  Inherited Create(Owner);
  FBmpBuffer := TBitmap.Create;
  FLight := TLedLight.create(self);
  FSupport := TLedSupport.create(self);
  Height := 13;
  Width := 13;
  FLight.ActiveColor := clLime;
  FLight.InactiveColor := clBlack;
  FLight.Width := 5;
  FLight.Height := 5;
  FLight.BorderStyle := bsNone;
  FSupport.BorderStyle := bsRaised;
  FSupport.ActiveColor := clGreen;
  FSupport.InactiveColor := clGray;
  FSupport.SetBounds(0, 0, Width, Height);
end;

destructor TrmoeBasicLed.Destroy;
begin
  FSupport.free;
  FLight.Free;
  FBmpBuffer.free;
  Inherited;
end;

procedure TrmoeBasicLed.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
begin
  Inherited ;
  If FBmpBuffer <> NIL then begin
    FBmpBuffer.Destroy;
    FBmpBuffer := TBitmap.Create;
    FBmpBuffer.Height := aHeight;
    FBmpBuffer.Width := aWidth;
  end;
  FSupport.Width := Width;
  FSupport.Height := Height;
end;

procedure TrmoeBasicLed.UpdateBmpBuffer;
var
  X1, Y1, X2, Y2: integer;
begin
  With FBmpBuffer.Canvas do begin

    FLight.FRefreshParentAfterChange := false; // avoid loop drawing reaction
    FSupport.FRefreshParentAfterChange := false;  // restore normal drawing
    FLight.Active := FActive;
    FSupport.Active := FActive;
    FLight.StandBy := FStandBy;
    FSupport.StandBy := FStandBy;
    if FStandBy then begin
      FSupport.BackGroundColor := FSupport.StandByColor;
      FLight.BackGroundColor := FSupport.StandByColor;
    end else begin
      if active then begin
        FSupport.BackGroundColor := FSupport.ActiveColor;
        FLight.BackGroundColor := FSupport.ActiveColor;
      end else begin
        FSupport.BackGroundColor := FSupport.InactiveColor;
        FLight.BackGroundColor := FSupport.ActiveColor;
      end;
    end;
    FSupport.paint;
    FLight.Paint;
    FSupport.FRefreshParentAfterChange := true;  // restore normal drawing
    FLight.FRefreshParentAfterChange := true;  // restore normal drawing

    // first, draw background, i.e. led support
    FBmpBuffer.Canvas.CopyRect(rect(0, 0, FSupport.FBmpBuffer.Width, FSupport.FBmpBuffer.Height),
                               FSupport.FBmpBuffer.Canvas,
                               rect(0, 0, FSupport.FBmpBuffer.Width, FSupport.FBmpBuffer.Height));

    // now draw electroluminescent part
    X1 := trunc((width - FLight.Width) / 2) + FLightHorzOffset;
    Y1 := trunc((height - FLight.height) / 2) + FLightVertOffset;
    X2 := X1 + FLight.Width;
    Y2 := Y1 + FLight.height;

    FBmpBuffer.Canvas.CopyRect(rect(X1, Y1, X2, Y2),
                               FLight.FBmpBuffer.Canvas,
                               rect(0, 0, FLight.FBmpBuffer.Width, FLight.FBmpBuffer.Height));
  end;
end;

procedure TrmoeBasicLed.Paint;
begin
  UpdateBmpBuffer;
  Canvas.Draw(0, 0, FBmpBuffer);
end;

procedure TrmoeBasicLed.SetActive(value: boolean);
begin
  if value <> FActive then
  begin
    FActive := value;
    FLight.Active := FActive;
    FSupport.Active := FActive;
    paint;
    //UpdateBmpBuffer;
  end;
end;

procedure TrmoeBasicLed.SetStandBy(value: boolean);
begin
  if value <> FStandBy then
  begin
    FStandBy := value;
    FLight.StandBy := FStandBy;
    FSupport.StandBy := FStandBy;
    paint;
    //UpdateBmpBuffer;
  end;
end;

procedure TrmoeBasicLed.SetLightHorzOffset(value: integer);
begin
  if value <> FLightHorzOffset then
  begin
    FLightHorzOffset := value;
    paint;
    //UpdateBmpBuffer;
  end;
end;

procedure TrmoeBasicLed.SetLightVertOffset(value: integer);
begin
  if value <> FLightVertOffset then
  begin
    FLightVertOffset := value;
    paint;
    //UpdateBmpBuffer;
  end;
end;


end.
